/**
 * \addtogroup RRKboard
 * @{
 */

/*****************************************************************************/
/**
 * \file RRKboard.c
 * Round robin task managed 4 x 4 keyboard routines for 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 09-07-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "RRKboard.h"

/*****************************************************************************
 *
 *                                Global Variables
 *
 *****************************************************************************/

/* 
 * Keyboard scan row/ column output and input pattern codes.
 */
static const unsigned char code KEY_IO[16] = {0x77, 0xB7, 0xD7, 0xE7,
                                              0x7B, 0xBB, 0xDB, 0xEB,
                                              0x7D, 0xBD, 0xDD, 0xED,
                                              0x7E, 0xBE, 0xDE, 0xEE};

/* 
 * Keyboard key position to ASCII representation codes.
 */
static const char code KEY_MAP[16] = {'1', '2', '3', 'A',
                                      '4', '5', '6', 'B',
                                      '7', '8', '9', 'C',
                                      '.', '0', '\r', '\b'}; // CR BS


/*****************************************************************************
 *
 *                            Private Function Prototypes 
 *
 *****************************************************************************/
static unsigned char rrkboard_scankey(void);


/*****************************************************************************
 *
 *                         Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                               rrkboard_scankey() 
 *
 *****************************************************************************/
static unsigned char rrkboard_scankey(void)
{
    /*
     * Scan each row of 4 x 4 matrix keyboard. If a key press is detected
     * return the keys corresponding position. Otherwise return
     * RRKBOARD_NOKEY (0x10).
     */

    unsigned char key = 0xFF;
    bit keyfound = 0;

    do
    {
        key++;

        /* Clear lower nibble of port.  */
        RRKBOARD_KEYPORT &= 0xF0;                     

        /* Output lower nibble to port. */
        RRKBOARD_KEYPORT |= KEY_IO[key] & 0x0F;       

        /* Retain upper nibble key press pattern. */
        keyfound = ((RRKBOARD_KEYPORT & 0xF0) == (KEY_IO[key] & 0xF0));
        
        /* If no key detected and there are more rows to scan, scan next. */
    } while((key < RRKBOARD_NOKEY) && (!keyfound));

    return(key);    
}


/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                                rrkboard_init()  
 *
 *****************************************************************************/
void rrkboard_init(void)
{
    /*
     * Set all keyboard port bits as inputs.
     * Set Timer 0 mode not gated, internal clock, 16 bit, no auto reload.
     * Timer 0 will free run at RRKBOARD_T0_CLOCK Hz.
     * Start Timer 0 running.
     */

    RRKBOARD_KEYPORT = 0xFF; /* All port bits as input.                      */

    TMOD &= 0xF0;            /* Clear lower nibble of TMOD (T0 control bits) */
    TMOD |= 0x01;            /* Set bit 0 of TMOD (T0 16 bit mode)           */

    TR0 = 1;                 /* Set Timer 0 run control bit in TCON          */
}

/*****************************************************************************
 *
 *                               rrkboard_getkey() 
 *
 *****************************************************************************/
char rrkboard_getkey(void)
{
    /*
     * Function to be used by polling to return an ASCII keycode only if a key
     * has been pressed consistently for in excess of RRKBOARD_DEBOUNCE_PERIOD
     * and more than RRKBOARD_KEYCOUNT times and to return that value only once
     * per keystroke of the user. If no key is detected then RRKBOARD_NOKEY is
     * returned. 
     */

           unsigned char key_sensed;
    static unsigned char key_retained;
    static unsigned int  time_base;
    static unsigned int  poll_cntr = 0;
    static unsigned char key_state = RRKBOARD_STATE_IDLE;


    /* Scan keyboard and save returned value. */
    key_sensed = rrkboard_scankey();
    
    switch (key_state)
    {
        case RRKBOARD_STATE_IDLE:
        {
            /* 
             * No key had previously been pressed.
             */

            if (key_sensed != RRKBOARD_NOKEY)
            {
                /*
                 * A first key press has now been detected.
                 */

                /* Remember which key was pressed and when. */
                key_retained = key_sensed;
                time_base = ((unsigned char)TH0 << 8) + TL0;

                /* Was first of RRKBOARD_KEYCOUNT times it must be pressed. */
                poll_cntr++;

                /* Move to next state. */
                key_state = RRKBOARD_STATE_KEYDOWN;
            }
        }
        break;

        case RRKBOARD_STATE_KEYDOWN:
        {
            /*
             * Key closure has previously been detected.
             */

            if (key_sensed != RRKBOARD_NOKEY)
            {
                /*
                 * A valid keycode has again been detected.
                 */

                if (key_retained == key_sensed)
                {
                    /*
                     * This key press is the same as the last one.
                     */

                    /* Advance consecutive identical keypress counter. */
                    poll_cntr++;

                    if ((poll_cntr > RRKBOARD_KEYCOUNT) 
					   && ((((unsigned int)TH0 << 8) + TL0 - time_base)
			               > RRKBOARD_DEBOUNCE_CYCLES))
                    {
                        /*
                         * Same key press has been detected enough times
                         * and for long enough.
                         */

                        /* Move to key opening wait state. */
                        key_state = RRKBOARD_STATE_WAIT_KEYUP;

                        return (KEY_MAP[key_retained]);
                    }
                }
                else
                {
                    /*
                     * This key press is not the same as the last one.
                     */

                    /* Restart consecutive identical keypress counter. */
                    poll_cntr = 1;

                    /* Remember which key was pressed this time and when. */
                    key_retained = key_sensed;
                    time_base = (unsigned char)(TH0 << 8) + TL0;
                }
            } 
            else
            {
                /*
                 * No valid keycode has been detected this time.
                 */

                /* Restart key press detection state machine. */
                poll_cntr = 0;
                key_state = RRKBOARD_STATE_IDLE;
            }
        }
        break;

        case RRKBOARD_STATE_WAIT_KEYUP:
        {
            /* 
             * A valid key press has been reported, now wait for release.
             */

            if (key_sensed == RRKBOARD_NOKEY)
            {
                /*
                 * All keys are open now.
                 */

                /* Restart key press detection state machine. */
                poll_cntr = 0;
                key_state = RRKBOARD_STATE_IDLE;
            }
        }
        break;
    }

    return(RRKBOARD_NOKEY);
}

/** @} */
