/**
 * \addtogroup RRKboard
 * @{
 */

/*****************************************************************************/
/**
 * \file RRKboard.h
 * Round robin task managed 4 x 4 keyboard routines for 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 09-07-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef RRKBOARD_H
#define RRKBOARD_H

#include "REG51.h"
 
/*****************************************************************************
 *
 *                               Constant Definitions.
 *
 *****************************************************************************/
 /** 
 * Hardware crystal frequency in Hz.
 * Adjust \b RRKBOARD_XTAL to match hardware.
 * \hideinitializer
 */
#define RRKBOARD_XTAL              18432000    

/*
 * Timer 0 mode 1 clock rate. (CPU non X2 mode)
 */
#define RRKBOARD_T0_CLOCK          RRKBOARD_XTAL / 12   

/** 
 * Hardware keyboard port assignment.
 * Adjust \b RRKBOARD_KEYPORT to match hardware.
 * \hideinitializer
 */
#define RRKBOARD_KEYPORT           P0

/**
 * Keyboard debounce period in seconds.
 * Adjust \b RRKBOARD_DEBOUNCE_PERIOD to required debounce duration.
 * \hideinitializer
 */
#define RRKBOARD_DEBOUNCE_PERIOD   0.010   

/**
 * Keyboard debounce counter cycles.
 * \b RRKBOARD_DEBOUNCE_CYCLES represents the number of timer 0 cycles
 * which must pass before the required debounce delay has elapsed.
 * \hideinitializer
 */
#define RRKBOARD_DEBOUNCE_CYCLES   (RRKBOARD_DEBOUNCE_PERIOD * RRKBOARD_T0_CLOCK)

/**
 * Keyboard keycount assignment.
 * \b RRKBOARD_KEYCOUNT represents the number of consecutive round robin
 * check-in iterations that a key must be depressed during for that 
 * key press to be recognised.
 * \hideinitializer
 */
#define RRKBOARD_KEYCOUNT          3


/*
 * Key press absent return value.
 * rrkboard_getkey() returns this when no key is pressed.
 * \hideinitializer
 */
#define RRKBOARD_NOKEY             16

/*
 * Key press debounce state machine state constants.
 */
#define RRKBOARD_STATE_IDLE        0
#define RRKBOARD_STATE_KEYDOWN     1
#define RRKBOARD_STATE_WAIT_KEYUP  2

/*****************************************************************************
 *
 *                           Public Function Prototypes.
 *
 *****************************************************************************/

/** 
 * 4 x 4 keyboard initialisation function.
 * Call early to setup port and timer hardware.
 * \param -
 * \return -
 */
extern void rrkboard_init(void);

/** 
 * 4 x 4 keyboard key press retrieval function.
 * Poll continuously for keyboard activity.
 * \param -
 * \return An \b unsigned \b char representing the ASCII value of the key
 * pressed or \b RRKBOARD_NOKEY if no key has been activated.
 */
extern char rrkboard_getkey(void);

#endif

/** @} */
